@@ -1,9 +1,13 @@
-from webthing import (Value, Property, Thing, SingleThing, WebThingServer)
 import tornado.ioloop
 import logging
+from webthing import (Value, Property, Thing, SingleThing, WebThingServer)
+from typing import List
 from openhab_pythonrule_engine.rule_engine import RuleEngine
 
 
+logging = logging.getLogger(__name__)
+
+
 class RuleEngineThing(Thing):
 
     def __init__(self, description: str, rule_engine: RuleEngine):
@@ -37,7 +41,7 @@ class RuleEngineThing(Thing):
                      metadata={
                          'title': 'loaded modules',
                          'type': 'string',
-                         'description': 'the list of loaded modules',
+                         'description': 'list of loaded modules',
                          'readOnly': True
                      }))
 
@@ -49,7 +53,7 @@ class RuleEngineThing(Thing):
                      metadata={
                          'title': 'last executed rule',
                          'type': 'string',
-                         'description': 'the last executed rule',
+                         'description': 'list of last executed rule',
                          'readOnly': True
                      }))
 
@@ -61,7 +65,7 @@ class RuleEngineThing(Thing):
                      metadata={
                          'title': 'last failed rule',
                          'type': 'string',
-                         'description': 'the failed executed rule',
+                         'description': 'list of failed executed rule',
                          'readOnly': True
                      }))
 
@@ -71,11 +75,42 @@ class RuleEngineThing(Thing):
     def on_update(self):
         self.ioloop.add_callback(self.__handle)
 
-    def __handle(self):
-        self.last_executed.notify_of_external_update(self.rule_engine.last_executed)
-        self.last_failed.notify_of_external_update(self.rule_engine.last_error)
-        self.loaded_modules.notify_of_external_update(", ".join(sorted(list(self.rule_engine.loaded_modules))))
+    def __print_module_info(self) -> List[str]:
+        func_info = {}
+        for rule in self.rule_engine.rules():
+            key = rule.module + "#" + rule.function_name
+            expressions = func_info.get(key, set())
+            expressions.add(rule.expression)
+            func_info[key] = expressions
+        sorted_keys = sorted(list(func_info.keys()))
+        return [key + " (" + ", ".join(func_info[key]) + ")" for key in sorted_keys]
 
+    def __print_last_executed(self) -> List[str]:
+        func_info = {}
+        for rule in self.rule_engine.rules():
+            if rule.last_executed is not None:
+                key = rule.module + "#" + rule.function_name
+                execution_times = func_info.get(key, [])
+                execution_times.append(rule.last_executed)
+                func_info[key] = execution_times
+        sorted_keys = sorted(list(func_info.keys()))
+        return [key + " (" + sorted(func_info[key])[-1].strftime("%H:%M:%S") + ")" for key in sorted_keys]
+
+    def __print_last_failed(self) -> List[str]:
+        func_info = {}
+        for rule in self.rule_engine.rules():
+            if rule.last_failed is not None:
+                key = rule.module + "#" + rule.function_name
+                execution_times = func_info.get(key, [])
+                execution_times.append(rule.last_failed)
+                func_info[key] = execution_times
+        sorted_keys = sorted(list(func_info.keys()))
+        return [key + " (" + sorted(func_info[key])[-1].strftime("%H:%M:%S") + ")" for key in sorted_keys]
+
+    def __handle(self):
+        self.last_executed.notify_of_external_update(", ".join(self.__print_last_executed()))
+        self.last_failed.notify_of_external_update(", ".join(self.__print_last_failed()))
+        self.loaded_modules.notify_of_external_update(", ".join(self.__print_module_info()))
 
 def run_server(port: int, description: str, rule_engine: RuleEngine):
     rule_engine_webthing = RuleEngineThing(description, rule_engine)
